@@ -23,9 +23,11 @@ perldl or pdl2 shell.
 
 package PDL::Dbg;
 
+use strict;
+use warnings;
 # used by info
-$PDL::Dbg::Title = "Type   Dimension       Flow  State          Mem";
-$PDL::Dbg::Infostr = "%6T %-15D  %3F   %-5S  %12M";
+our $Title = "Type   Dimension       Flow  State          Mem";
+our $Infostr = "%6T %-15D  %3F   %-5S  %12M";
 
 package PDL;
 
@@ -35,7 +37,7 @@ package PDL;
 
 =for ref
 
-Print info about a piddle (or all known piddles)
+Print info about an ndarray (or all known ndarrays)
 
 =for example
 
@@ -47,7 +49,7 @@ Print info about a piddle (or all known piddles)
     PDL (52433464) Type: Double
 
 
-This function prints some information about piddles. It can be invoked
+This function prints some information about ndarrays. It can be invoked
 as a class method (e.g. C<PDL-E<gt>px> ) or as an instance method (e.g.
 C<$pdl-E<gt>px($arg)>). If
 
@@ -55,7 +57,7 @@ C<$pdl-E<gt>px($arg)>). If
 
 =item invoked as a class method
 
-it prints info about all piddles found in the current package
+it prints info about all ndarrays found in the current package
 (I<excluding> C<my> variables). This comes in quite handy when you are
 not quite sure which pdls you have already defined, what data they
 hold , etc. C<px> is supposed to support inheritance and prints info
@@ -69,7 +71,7 @@ The default package is that of the caller.
 
 =item invoked as an instance method
 
-it prints info about that particular piddle if C<$PDL::debug> is
+it prints info about that particular ndarray if C<$PDL::debug> is
 true and returns the pdl object upon completion. It accepts an
 optional string argument that is simply prepended to the default info
 if it doesn't contain a C<%> character. If, however, the argument
@@ -112,27 +114,22 @@ sub px {
   my $classname = $arg;
   # find the correct package
   $package .= "::" unless $package =~ /::$/;
-  *stab = *{"main::"};
-  while ($package =~ /(\w+?::)/g){
-    *stab = $ {stab}{$1};
-  }
+  my $stab = \%main::;
+  $stab = $stab->{$_.'::'} for grep length, split /::/, $package;
   print "$classname variables in package $package\n\n";
   my $title = "Name         $PDL::Dbg::Title\n";
   print $title;
   print '-'x(length($title)+3)."\n";
-  my ($pdl,$npdls,$key,$val,$info) = ((),0,"","","");
-  # while (($key,$val) = each(%stab)) {
-  foreach $key ( sort { lc($a) cmp lc($b) } keys(%stab) ) {
-     $val = $stab{$key};
-     $pdl = ${"$package$key"};
+  my $npdls = 0;
+  foreach my $key ( sort { lc($a) cmp lc($b) } keys %$stab ) {
+     my $pdl = do { no strict 'refs'; ${"$package$key"} };
+     next if !UNIVERSAL::isa($pdl,$classname);
      # print info for all objects derived from this class
-     if (UNIVERSAL::isa($pdl,$classname)) {
-        $npdls++;
-        $info = $pdl->info($PDL::Dbg::Infostr);
-        printf "\$%-11s %s %s\n",$key,$info,(ref($pdl) eq $classname ? '' :
-           ref($pdl));
-        # also print classname for derived classes
-     }
+     $npdls++;
+     my $info = $pdl->info($Infostr);
+     printf "\$%-11s %s %s\n",$key,$info,(ref $pdl eq $classname ? '' :
+        ref $pdl);
+     # also print classname for derived classes
   }
   print "no $classname objects in package $package\n"
        unless $npdls;
