@@ -1,6 +1,6 @@
 =head1 NAME
 
-PDL::Install::Files
+PDL::Install::Files -- Module for use by L<ExtUtils::Depends> and L<Inline>
 
 =head1 SYNOPSIS
 
@@ -17,22 +17,24 @@ no user-serviceable parts inside.
 =cut
 
 package PDL::Install::Files;
+use strict;
+use warnings;
 # support ExtUtils::Depends
 require PDL::Core::Dev;
 
 our $VERSION = '2.009';
 
-$self = {
+my $self = {
   'typemaps' => [ &PDL::Core::Dev::PDL_TYPEMAP ],
   'inc' => &PDL::Core::Dev::PDL_INCLUDE,
   'libs' => '',
   'deps' => [],
 };
-@deps = @{ $self->{deps} };
-@typemaps = @{ $self->{typemaps} };
-$libs = $self->{libs};
-$inc = $self->{inc};
-$CORE = undef;
+my @deps = @{ $self->{deps} };
+my @typemaps = @{ $self->{typemaps} };
+my $libs = $self->{libs};
+my $inc = $self->{inc};
+my $CORE = undef;
 foreach (@INC) {
   if ( -f "$_/PDL/Install/Files.pm") { $CORE = $_ . "/PDL/Install/"; last; }
 }
@@ -45,7 +47,6 @@ require Inline;
 sub Inline {
   my ($class, $lang) = @_;
   return {} if $lang eq 'Pdlpp';
-  return unless $lang eq 'C';
   unless($ENV{"PDL_Early_Inline"} // ($Inline::VERSION >= 0.68) ) {
       die "PDL::Inline: requires Inline version 0.68 or higher to make sense\n  (yours is $Inline::VERSION). You should upgrade Inline, \n   or else set \$ENV{PDL_Early_Inline} to a true value to ignore this message.\n";
   }
