@@ -9,45 +9,14 @@ our %INVALID_PAR = map +($_=>1), qw(
   I
 );
 
-# split regex $re separated arglist
-# but ignore bracket-protected bits
-# (i.e. text that is within matched brackets)
-my $prebrackreg = qr/^([^\(\{\[]*)/;
-sub splitprotected ($$) {
-  require Text::Balanced;
-  my ($re,$txt) = @_;
-  return () if !defined $txt || $txt =~ /^\s*$/;
-  my ($got,$pre) = (1,'');
-  my @chunks = ('');
-  my $ct = 0; # infinite loop protection
-  while ($got && $txt =~ /[({\[]/ && $ct++ < 1000) {
-    # print "iteration $ct\n";
-    ($got,$txt,$pre) =
-      Text::Balanced::extract_bracketed($txt,'{}()[]',$prebrackreg);
-    my @partialargs = split $re, $pre, -1;
-    $chunks[-1] .= shift @partialargs if @partialargs;
-    push @chunks, @partialargs;
-    $chunks[-1] .= $got;
-  }
-  confess "possible infinite parse loop, splitting '$txt' "
-			   if $ct >= 1000;
-  my @partialargs = split $re, $txt, -1;
-  $chunks[-1] .= shift @partialargs if @partialargs;
-  push @chunks, @partialargs if @partialargs;
-  # print STDERR "args found: $#chunks\n";
-  # print STDERR "splitprotected $txt on $re: [",join('|',@chunks),"]\n";
-  return @chunks;
-}
-
 my $typeregex = join '|', map $_->ppforcetype, types;
 my $complex_regex = join '|', qw(real complex);
+our $sqbr_re = qr/\[([^]]*)\]/x;
 our $pars_re = qr/^
 	\s*(?:($complex_regex|$typeregex)\b([+]*)|)\s*	# $1,2: first option then plus
-	(?:
-	\[([^]]*)\]	# $3: The initial [option] part
-	)?\s*
-	(\w+)	  	# $4: The name
-	\(([^)]*)\)	# $5: The indices
+	(?:$sqbr_re)?\s*	# $3: The initial [option] part
+	(\w+)			# $4: The name
+	\(([^)]*)\)		# $5: The indices
 /x;
 my %flag2info = (
   io => [[qw(FlagW)]],
@@ -81,14 +50,14 @@ sub new {
 	# originally defined here, but were moved to PDL::PP for FullDoc parsing.
 	$string =~ $pars_re
 		 or confess "Invalid pdl def $string (regex $pars_re)\n";
-	my($opt1,$opt_plus,$opt2,$name,$inds) = map $_ // '', ($1,$2,$3,$4,$5);
-	print "PDL: '$opt1$opt_plus', '$opt2', '$name', '$inds'\n"
+	my($opt1,$opt_plus,$sqbr_opt,$name,$inds) = map $_ // '', $1,$2,$3,$4,$5;
+	print "PDL: '$opt1$opt_plus', '$sqbr_opt', '$name', '$inds'\n"
 		  if $::PP_VERBOSE;
 	croak "Invalid Pars name: $name"
 	  if $INVALID_PAR{$name};
 # Set my internal variables
 	$this->{Name} = $name;
-	$this->{Flags} = [(split ',',$opt2),($opt1?$opt1:())];
+	$this->{Flags} = [(split ',',$sqbr_opt),($opt1?$opt1:())];
 	for(@{$this->{Flags}}) {
 		confess("Invalid flag $_ given for $string\n")
 			unless my ($set, $store) = @{ $flag2info{$_} || [] };
@@ -135,7 +104,7 @@ sub add_inds {
 }
 
 
-# do the dimension checking for perl level threading
+# do the dimension checking for perl level broadcasting
 # assumes that IndObjs have been created
 sub perldimcheck {
   my ($this,$pdl) = @_;
@@ -144,7 +113,7 @@ sub perldimcheck {
   return 1 if $pdl->isnull;
   my $rdims = @{$this->{RawInds}};
   croak ("not enough dimensions for ".$this->name)
-    if ($pdl->threadids)[0] < $rdims;
+    if ($pdl->broadcastids)[0] < $rdims;
   my @dims = $pdl->dims;
   my ($i,$ind) = (0,undef);
   for $ind (@{$this->{IndObjs}}) {
@@ -220,8 +189,8 @@ sub do_access {
 	my $pdl = $this->{Name};
 # Parse substitutions into hash
 	my %subst = map
-	 {/^\s*(\w+)\s*=>\s*(\S*)\s*$/ or confess "Invalid subst $_\n"; ($1,$2)}
-	 	splitprotected ',',$inds;
+	 {/^\s*(\w+)\s*=>\s*(\S*)\s*$/ or confess "Invalid subst $_ in ($inds) (no spaces in => value)\n"; ($1,$2)}
+		PDL::PP::Rule::Substitute::split_cpp($inds);
 # Generate the text
 	my $text;
 	$text = "(${pdl}_datap)"."[";
@@ -275,12 +244,12 @@ sub do_indterm { my($this,$pdl,$ind,$subst,$context) = @_;
 }
 
 sub get_xsdatapdecl { 
-    my($this,$ctype) = @_;
+    my($this,$ctype,$nulldatacheck) = @_;
     my $pdl = $this->get_nname;
     my $flag = $this->get_nnflag;
     my $name = $this->{Name};
     my $macro = "PDL_DECLARE_PARAMETER".($this->{BadFlag} ? "_BADVAL" : "");
-    "$macro($ctype, $flag, $name, $pdl)";
+    "$macro($ctype, $flag, $name, $pdl, $nulldatacheck)";
 }
 
 1;
