@@ -1,32 +1,32 @@
 =head1 NAME
 
-PDL::Graphics::TriD -- PDL 3D interface
+PDL::Graphics::TriD - PDL 3D interface
 
 =head1 SYNOPSIS
 
  use PDL::Graphics::TriD;
- 
+
  # Generate a somewhat interesting sequence of points:
  $t = sequence(100)/10;
  $x = sin($t); $y = cos($t), $z = $t;
- $coords = cat($x, $y, $z)->xchg(0,1);
+ $coords = cat($x, $y, $z)->transpose;
  my $red = cos(2*$t); my $green = sin($t); my $blue = $t;
- $colors = cat($red, $green, $blue)->xchg(0,1);
- 
+ $colors = cat($red, $green, $blue)->transpose;
+
  # After each graph, let the user rotate and
- # wait for him to press 'q', then make new graph
+ # wait for them to press 'q', then make new graph
  line3d($coords);       # $coords = (3,n,...)
  line3d($coords,$colors);  # $colors = (3,n,...)
  line3d([$x,$y,$z]);
- 
+
  # Generate a somewhat interesting sequence of surfaces
  $surf1 = (rvals(100, 100) / 50)**2 + sin(xvals(100, 100) / 10);
  $surf2 = sqrt(rvals(zeroes(50,50))/2);
  $x = sin($surface); $y = cos($surface), $z = $surface;
- $coords = cat($x, $y, $z)->xchg(0,1);
+ $coords = cat($x, $y, $z)->transpose;
  $red = cos(2*$surface); $green = sin($surface); $blue = $surface;
- $colors = cat($red, $green, $blue)->xchg(0,1);
- imagrgb([$red,$green,$blue]);     # 2-d piddles
+ $colors = cat($red, $green, $blue)->transpose;
+ imagrgb([$red,$green,$blue]);     # 2-d ndarrays
  lattice3d([$surf1]);
  points3d([$x,$y,$z]);
  spheres3d([$x,$y,$z]);  # preliminary implementation
@@ -34,11 +34,10 @@ PDL::Graphics::TriD -- PDL 3D interface
  hold3d(); # the following graphs are on top of each other and the previous
  line3d([$x,$y,$z]);
  line3d([$x,$y,$z+1]);
- $pic = grabpic3d(); # Returns the picture in a (3,$x,$y) float piddle (0..1).
+ $pic = grabpic3d(); # Returns the picture in a (3,$x,$y) float ndarray (0..1).
 
  release3d(); # the next graph will again wipe out things.
 
-
 =head1 WARNING
 
 These modules are still in a somewhat unfocused state: don't use them yet
@@ -100,7 +99,7 @@ PDL and wait for the next PDL image over the network.
 
 Specifying a set of coordinates is generally a context-dependent operation.
 For a traditional 3D surface plot, you'll want two of the coordinates
-to have just the xvals and yvals of the piddle, respectively.
+to have just the xvals and yvals of the ndarray, respectively.
 For a line, you would generally want to have one coordinate held
 at zero and the other advancing.
 
@@ -110,19 +109,19 @@ interpretation.
 
 The alternative syntaxes for specifying a set of coordinates (or colors) are
 
-   $piddle                             # MUST have 3 as first dim.
-  [$piddle]
-  [$piddle1,$piddle2]
-  [$piddle1,$piddle2,$piddle3]
-  [CONTEXT,$piddle]
-  [CONTEXT,$piddle1,$piddle2]
-  [CONTEXT,$piddle1,$piddle2,$piddle3]
+   $ndarray                             # MUST have 3 as first dim.
+  [$ndarray]
+  [$ndarray1,$ndarray2]
+  [$ndarray1,$ndarray2,$ndarray3]
+  [CONTEXT,$ndarray]
+  [CONTEXT,$ndarray1,$ndarray2]
+  [CONTEXT,$ndarray1,$ndarray2,$ndarray3]
 
 where C<CONTEXT> is a string describing in which context you wish these
-piddles to be interpreted. Each routine specifies a default context
+ndarrays to be interpreted. Each routine specifies a default context
 which is explained in the routines documentation.
 Context is usually used only to understand what the user wants
-when he/she specifies less than 3 piddles.
+when they specify less than 3 ndarrays.
 
 The following contexts are currently supported:
 
@@ -130,25 +129,25 @@ The following contexts are currently supported:
 
 =item SURF2D
 
-A 2-D lattice. C< [$piddle] > is interpreted as the Z coordinate over
+A 2-D lattice. C< [$ndarray] > is interpreted as the Z coordinate over
 a lattice over the first dimension. Equivalent to
-C<< [$piddle->xvals, $piddle->yvals, $piddle] >>.
+C<< [$ndarray->xvals, $ndarray->yvals, $ndarray] >>.
 
 =item POLAR2D
 
-A 2-D polar coordinate system. C< [$piddle] > is interpreted as the
-z coordinate over theta and r (theta = the first dimension of the piddle).
+A 2-D polar coordinate system. C< [$ndarray] > is interpreted as the
+z coordinate over theta and r (theta = the first dimension of the ndarray).
 
 =item COLOR
 
-A set of colors. C< [$piddle] > is interpreted as grayscale color
-(equivalent to C< [$piddle,$piddle,$piddle] >).
+A set of colors. C< [$ndarray] > is interpreted as grayscale color
+(equivalent to C< [$ndarray,$ndarray,$ndarray] >).
 
 =item LINE
 
-A line made of 1 or 2 coordinates. C< [$piddle] > is interpreted as
-C<< [$piddle->xvals,$piddle,0] >>. C< [$piddle1,$piddle2] > is interpreted as
-C<< [$piddle1,$piddle2,$piddle1->xvals] >>.
+A line made of 1 or 2 coordinates. C< [$ndarray] > is interpreted as
+C<< [$ndarray->xvals,$ndarray,0] >>. C< [$ndarray1,$ndarray2] > is interpreted as
+C<< [$ndarray1,$ndarray2,$ndarray1->xvals] >>.
 
 =back
 
@@ -165,7 +164,7 @@ will do exactly the same.
 
 =head2 Wrapping your head around 3d surface specifications
 
-Let's begin by thnking about how you might make a 2d data plot.
+Let's begin by thinking about how you might make a 2d data plot.
 If you sampled your data at regular intervals, you would have
 a time serires y(t) = (y0, y1, y2, ...).  You could plot y vs t
 by computing t0 = 0, t1 = dt, t2 = 2 * dt, and then plotting
@@ -188,25 +187,18 @@ C<mesh3d([$surface])>.
 Of course, your data is not required to be regularly gridded.
 You could, for example, be measuring the flight path of a bat
 flying after mosquitos, which could be wheeling and arching
-all over the space.  This is what you might plot using 
+all over the space.  This is what you might plot using
 C<line3d([$x, $y, $z])>.  You could plot the trajectories of
 multiple bats, in which case C<$x>, C<$y>, and C<$z> would have
 multiple columns, but in general you wouldn't expect them to be
 coordinated.
 
-Finally, imagine that you have an air squadron flying in
-formation.  Your (x, y, z) data is not regularly gridded, but
-the (x, y, z) data for each plane should be coordinated and
-we can imagine that their flight path sweep out a surface.
-We could draw this data using C<line3d([$x, $y, $z])>, where
-each column in the variables corresponds to a different plane,
-but it would also make sense to draw this data using
-C<mesh3d([$x, $y, $z])>, since the planes' proximity to each
-other should be fairly consistent.  In other words, it makes
-sense to think of the planes as sweeping out a coordinated
-surface, which C<mesh3d> would draw for you, whereas you would
-not expect the trajectories of the various bats to describe a
-meaningful surface (unless you're into fractals, perhaps).
+More generally, each coordinate is expected to be arranged in a 3D
+fashion, similar to C<3,x,y>. The "3" is the actual 3D coordinates of
+each point. The "x,y" help with gridding, because each point at C<x,y>
+is expected to have as geographical neighbours C<x+1,y>, C<x-1,y>,
+C<x,y+1>, C<x,y-1>, and the grid polygon-building relies on that.
+This is how, and why, the 3D earth in C<demo 3d> arranges its data.
 
  #!/usr/bin/perl
 
@@ -221,11 +213,11 @@ meaningful surface (unless you're into fractals, perhaps).
  print "using line3d to plot a trajectory (press q when you're done twiddling)\n";
  line3d [$x,$y,$z];
 
- # If you give it a single piddle, it expects
+ # If you give it a single ndarray, it expects
  # the data to look like
  # ((x1, y1, z1), (x2, y2, z2), ...)
  # which is why we have to do the exchange:
- $coords = cat($x, $y, $z)->xchg(0,1);
+ $coords = cat($x, $y, $z)->transpose;
  print "again, with a different coordinate syntax (press q when you're done twiddling)\n";
  line3d $coords;
 
@@ -238,8 +230,8 @@ meaningful surface (unless you're into fractals, perhaps).
 
  # Draw a mobius strip:
  $two_pi = 8 * atan2(1,1);
- $t = sequence(50) / 50 * $two_pi;
- # We want two paths:
+ $t = sequence(51) / 50 * $two_pi;
+ # We want three paths:
  $mobius1_x = cos($t) + 0.5 * sin($t/2);
  $mobius2_x = cos($t);
  $mobius3_x = cos($t) - 0.5 * sin($t/2);
@@ -265,7 +257,6 @@ meaningful surface (unless you're into fractals, perhaps).
  print "The same mobius strip using imag3d\n";
  imag3d $mobius_surface, {Lines => 0};
 
-
 =head1 SIMPLE ROUTINES
 
 Because using the whole object-oriented interface for doing
@@ -280,9 +271,11 @@ routines are supported:
 
 3D line plot, defined by a variety of contexts.
 
+Implemented by C<PDL::Graphics::TriD::LineStrip>.
+
 =for usage
 
- line3d piddle(3,x), {OPTIONS}
+ line3d ndarray(3,x), {OPTIONS}
  line3d [CONTEXT], {OPTIONS}
 
 =for example
@@ -308,10 +301,14 @@ contexts and options
 
 3D rendered image plot, defined by a variety of contexts
 
+Implemented by C<PDL::Graphics::TriD::SLattice_S>.
+
+The variant, C<imag3d_ns>, is implemented by C<PDL::Graphics::TriD::SLattice>.
+
 =for usage
 
- imag3d piddle(3,x,y), {OPTIONS}
- imag3d [piddle,...], {OPTIONS}
+ imag3d ndarray(3,x,y), {OPTIONS}
+ imag3d [ndarray,...], {OPTIONS}
 
 =for example
 
@@ -330,10 +327,12 @@ contexts and options
 
 3D mesh plot, defined by a variety of contexts
 
+Implemented by C<PDL::Graphics::TriD::Lattice>.
+
 =for usage
 
- mesh3d piddle(3,x,y), {OPTIONS}
- mesh3d [piddle,...], {OPTIONS}
+ mesh3d ndarray(3,x,y), {OPTIONS}
+ mesh3d [ndarray,...], {OPTIONS}
 
 =for example
 
@@ -356,16 +355,36 @@ contexts and options
 
 alias for mesh3d
 
+=head2 trigrid3d
+
+Show a triangular mesh, giving C<$vertices> and C<$faceidx> which is
+a series of triplets of indices into the vertices, each describing
+one triangle. The order of points matters for the shading - the normal
+vector points towards the clockface if the points go clockwise.
+
+Options: C<Smooth> (on by default), C<Lines> (off by default),
+C<ShowNormals> (off by default, useful for debugging).
+
+Implemented by C<PDL::Graphics::TriD::STrigrid_S>.
+
+=head2 trigrid3d_ns
+
+Like L</trigrid3d>, but without shading or normals.
+
+Implemented by C<PDL::Graphics::TriD::STrigrid>.
+
 =head2 points3d
 
 =for ref
 
 3D points plot, defined by a variety of contexts
 
+Implemented by C<PDL::Graphics::TriD::Points>.
+
 =for usage
 
- points3d piddle(3), {OPTIONS}
- points3d [piddle,...], {OPTIONS}
+ points3d ndarray(3), {OPTIONS}
+ points3d [ndarray,...], {OPTIONS}
 
 =for example
 
@@ -383,22 +402,24 @@ contexts and options
 
 3D spheres plot (preliminary implementation)
 
+This is a preliminary implementation as a proof of
+concept.  It has fixed radii for the spheres being
+drawn and no control of color or transparency.
+
+Implemented by C<PDL::Graphics::TriD::Spheres>.
+
 =for usage
 
- spheres3d piddle(3), {OPTIONS}
- spheres3d [piddle,...], {OPTIONS}
+ spheres3d ndarray(3), {OPTIONS}
+ spheres3d [ndarray,...], {OPTIONS}
 
 =for example
 
 Example:
 
- pdl> spheres3d ndcoords(10,10,10)->clump(1,2,3)  
- 
- - lattice of spheres at coordinates on 10x10x10 grid
+ pdl> spheres3d ndcoords(10,10,10)->clump(1,2,3)
 
-This is a preliminary implementation as a proof of
-concept.  It has fixed radii for the spheres being
-drawn and no control of color or transparency.
+ - lattice of spheres at coordinates on 10x10x10 grid
 
 =head2 imagrgb
 
@@ -406,10 +427,12 @@ drawn and no control of color or transparency.
 
 2D RGB image plot (see also imag2d)
 
+Implemented by C<PDL::Graphics::TriD::Image>.
+
 =for usage
 
- imagrgb piddle(3,x,y), {OPTIONS}
- imagrgb [piddle,...], {OPTIONS}
+ imagrgb ndarray(3,x,y), {OPTIONS}
+ imagrgb [ndarray,...], {OPTIONS}
 
 This would be used to plot an image, specifying
 red, green and blue values at each point. Note:
@@ -429,14 +452,16 @@ e.g.
 
 2D RGB image plot as an object inside a 3D space
 
+Implemented by C<PDL::Graphics::TriD::Image>.
+
 =for usage
 
- imagrdb3d piddle(3,x,y), {OPTIONS}
- imagrdb3d [piddle,...], {OPTIONS}
+ imagrdb3d ndarray(3,x,y), {OPTIONS}
+ imagrdb3d [ndarray,...], {OPTIONS}
 
-The piddle gives the colors. The option allowed is Points,
+The ndarray gives the colors. The option allowed is Points,
 which should give 4 3D coordinates for the corners of the polygon,
-either as a piddle or as array ref.
+either as an ndarray or as array ref.
 The default is [[0,0,0],[1,0,0],[1,1,0],[0,1,0]].
 
 =for example
@@ -456,7 +481,7 @@ Grab a 3D image from the screen.
 
  $pic = grabpic3d();
 
-The returned piddle has dimensions (3,$x,$y) and is of type float
+The returned ndarray has dimensions (3,$x,$y) and is of type float
 (currently). XXX This should be altered later.
 
 =head2 hold3d, release3d
@@ -506,11 +531,12 @@ user explicitly presses 'q'.
  imag3d(..);
  nokeeptwiddling3d();
  $o = imag3d($c);
- while(1) {
- 	$c .= nextfunc($c);
- 	$o->data_changed();
- 	twiddle3d();		# animate one step, then return.
- }
+ do {
+	$c .= nextfunc($c);
+	$o->data_changed;
+ } while(!twiddle3d()); # animate one step, then iterate
+ keeptwiddling3d();
+ twiddle3d(); # wait one last time
 
 =head2 twiddle3d
 
@@ -519,12 +545,31 @@ user explicitly presses 'q'.
 Wait for the user to rotate the image in 3D space.
 
 Let the user rotate the image in 3D space, either for one step
-or until (s)he presses 'q', depending on the 'keeptwiddling3d'
+or until they press 'q', depending on the 'keeptwiddling3d'
 setting. If 'keeptwiddling3d' is not set the routine returns
 immediately and indicates that a 'q' event was received by
 returning 1. If the only events received were mouse events,
 returns 0.
 
+=head2 close3d
+
+=for ref
+
+Close the currently-open 3D window.
+
+=head1 NOT EXPORTED
+
+These functions are not exported, partly because they are not fully
+implemented.
+
+=over
+
+=item contour3d
+
+Implemented by C<PDL::Graphics::TriD::Contours>.
+
+=back
+
 =head1 CONCEPTS
 
 The key concepts (object types) of TriD are explained in the following:
@@ -561,20 +606,20 @@ your own risk.
 The syntax C<PDL::Graphics::TriD::Scale(x,y,z)> here means that you create
 an object like
 
-	$c = new PDL::Graphics::TriD::Scale($x,$y,$z);
+	$c = PDL::Graphics::TriD::Scale->new($x,$y,$z);
 
 =head2 PDL::Graphics::TriD::LineStrip
 
 This is just a line or a set of lines. The arguments are 3 1-or-more-D
-piddles which describe the vertices of a continuous line and an 
-optional color piddle (which is 1-D also and simply
+ndarrays which describe the vertices of a continuous line and an
+optional color ndarray (which is 1-D also and simply
 defines the color between red and blue. This will probably change).
 
 =head2 PDL::Graphics::TriD::Lines
 
 This is just a line or a set of lines. The arguments are 3 1-or-more-D
-piddles where each contiguous pair of vertices describe a line segment 
-and an optional color piddle (which is 1-D also and simply
+ndarrays where each contiguous pair of vertices describe a line segment
+and an optional color ndarray (which is 1-D also and simply
 defines the color between red and blue. This will probably change).
 
 =head2 PDL::Graphics::TriD::Image
@@ -586,12 +631,12 @@ be relatively memory and execution-time-friendly.
 =head2 PDL::Graphics::TriD::Lattice
 
 This is a 2-D set of points connected by lines in 3-space.
-The constructor takes as arguments 3 2-dimensional piddles.
+The constructor takes as arguments 3 2-dimensional ndarrays.
 
 =head2 PDL::Graphics::TriD::Points
 
 This is simply a set of points in 3-space. Takes as arguments
-the x, y and z coordinates of the points as piddles.
+the x, y and z coordinates of the points as ndarrays.
 
 =head2 PDL::Graphics::TriD::Scale(x,y,z)
 
@@ -620,40 +665,37 @@ to the previous viewport in the (0,1) range.
 Every implementation-level window object should implement the new_viewport
 method.
 
-=head1 EXAMPLE SCRIPT FOR VARIOUS 
-
 =cut
 
 #KGB: NEEDS DOCS ON COMMON OPTIONS!!!!!
 
 # List of global variables
-# 
+#
 # $PDL::Graphics::TriD::offline
-# $PDL::Graphics::TriD::Settings 
-# $PDL::Graphics::TriD::verbose
+# $PDL::Graphics::TriD::Settings
+$PDL::Graphics::TriD::verbose //= 0;
 # $PDL::Graphics::TriD::keeptwiddling
-# $PDL::Graphics::TriD::hold_on
-# $PDL::Graphics::TriD::curgraph
-# $PDL::Graphics::TriD::cur
+# $PDL::Graphics::TriD::only_one
 # $PDL::Graphics::TriD::create_window_sub
 # $PDL::Graphics::TriD::current_window
-# 
+#
 # '
 
-package PDL::Graphics::TriD::Basic;
 package PDL::Graphics::TriD;
 
+use strict;
+use warnings;
 use PDL::Exporter;
 use PDL::Core '';  # barf
-use vars qw/@ISA @EXPORT_OK %EXPORT_TAGS/;
-@ISA = qw/PDL::Exporter/;
-@EXPORT_OK = qw/imag3d_ns imag3d line3d mesh3d lattice3d points3d
+our @ISA = qw/PDL::Exporter/;
+our @EXPORT_OK = qw/imag3d_ns imag3d line3d mesh3d lattice3d points3d
+  trigrid3d trigrid3d_ns
   spheres3d describe3d imagrgb imagrgb3d hold3d release3d
-  keeptwiddling3d nokeeptwiddling3d
+  keeptwiddling3d nokeeptwiddling3d close3d
   twiddle3d grabpic3d tridsettings/;
-%EXPORT_TAGS = (Func=>[@EXPORT_OK]);
+our %EXPORT_TAGS = (Func=>\@EXPORT_OK);
+our $verbose;
 
-#use strict;
 use PDL::Graphics::TriD::Object;
 use PDL::Graphics::TriD::Window;
 use PDL::Graphics::TriD::ViewPort;
@@ -662,14 +704,12 @@ use PDL::Graphics::TriD::Quaternion;
 use PDL::Graphics::TriD::Objects;
 use PDL::Graphics::TriD::Rout;
 
-
 # Then, see which display method are we using:
 
+$PDL::Graphics::TriD::device = $PDL::Graphics::TriD::device;
 BEGIN {
-	my $dev;
-	$dev ||= $::PDL::Graphics::TriD::device; # First, take it from this variable.
+	my $dev = $PDL::Graphics::TriD::device; # First, take it from this variable.
 	$dev ||= $::ENV{PDL_3D_DEVICE};
-
         if(!defined $dev) {
 #            warn "Default PDL 3D device is GL (OpenGL):
 # Set PDL_3D_DEVICE=GL in your environment in order not to see this warning.
@@ -688,16 +728,15 @@ BEGIN {
 		(barf "Invalid PDL 3D device '$_' specified!");
 	}
 	my $mod = $dv;
-	$mod =~ s|::|//|g;
+	$mod =~ s|::|/|g;
 	print "dev = $dev mod=$mod\n" if($verbose);
- 
 	require "$mod.pm";
 	$dv->import;
         my $verbose;
 }
 
-
 # currently only used by VRML backend
+$PDL::Graphics::TriD::Settings = $PDL::Graphics::TriD::Settings;
 sub tridsettings {return $PDL::Graphics::TriD::Settings}
 
 # Allowable forms:
@@ -706,56 +745,50 @@ sub realcoords {
 	my($type,$c) = @_;
 	if(ref $c ne "ARRAY") {
 		if($c->getdim(0) != 3) {
-			barf "If one piddle given for coordinate, must be (3,...) or have default interpretation";
+			barf "If one ndarray given for coordinate, must be (3,...) or have default interpretation";
 		}
 		return $c ;
 	}
-	if(!ref $c->[0]) {$type = shift @$c}
-	if($#$c < 0 || $#$c>2) {
+	my @c = @$c;
+	if(!ref $c[0]) {$type = shift @c}
+	if(!@c || @c>3) {
 		barf "Must have 1..3 array members for coordinates";
 	}
-	if($#$c == 0 and $type =~ /^SURF2D$/) {
+	if(@c == 1 and $type eq "SURF2D") {
 		# surf2d -> this is z axis
-		@$c = ($c->[0]->xvals,$c->[0]->yvals,$c->[0]);
-	} elsif($#$c == 0 and $type eq "POLAR2D") {
-		my $t = 6.283 * $c->[0]->xvals / ($c->[0]->getdim(0)-1);
-		my $r = $c->[0]->yvals / ($c->[0]->getdim(1)-1);
-		@$c = ($r * sin($t), $r * cos($t), $c->[0]);
-	} elsif($#$c == 0 and $type eq "COLOR") {
-		# color -> 1 piddle = grayscale
-		@$c = ($c->[0], $c->[0], $c->[0]);
-	} elsif($#$c == 0 and $type eq "LINE") {
-		@$c = ($c->[0]->xvals, $c->[0], 0);
-	} elsif($#$c == 1 and $type eq "LINE") {
-		@$c = ($c->[0], $c->[1], $c->[0]->xvals);
+		@c = ($c[0]->xvals,$c[0]->yvals,$c[0]);
+	} elsif(@c == 1 and $type eq "POLAR2D") {
+		my $t = 6.283 * $c[0]->xvals / ($c[0]->getdim(0)-1);
+		my $r = $c[0]->yvals / ($c[0]->getdim(1)-1);
+		@c = ($r * sin($t), $r * cos($t), $c[0]);
+	} elsif(@c == 1 and $type eq "COLOR") {
+		# color -> 1 ndarray = grayscale
+		@c = @c[0,0,0];
+	} elsif(@c == 1 and $type eq "LINE") {
+		@c = ($c[0]->xvals, $c[0], 0);
+	} elsif(@c == 2 and $type eq "LINE") {
+		@c = (@c[0,1], $c[0]->xvals);
 	}
 	# XXX
-	if($#$c != 2) {
+	if(@c != 3) {
 		barf("Must have 3 coordinates if no interpretation (here '$type')");
 	}
 	# allow a constant (either pdl or not) to be introduced in one dimension
-        foreach(0..2){  
-	  if(ref($c->[$_]) ne "PDL" or $c->[$_]->nelem==1){
-	    $c->[$_] = $c->[$_]*(PDL->ones($c->[($_+1)%3]->dims));
+	foreach(0..2) {
+	  if(ref($c[$_]) ne "PDL" or $c[$_]->nelem==1){
+	    $c[$_] = $c[$_]*(PDL->ones($c[($_+1)%3]->dims));
 	  }
 	}
-	my $g = PDL->null;
-	&PDL::Graphics::TriD::Rout::combcoords(@$c,$g);
+	my $g = PDL::Graphics::TriD::Rout::combcoords(@c);
 	$g->dump if $PDL::Graphics::TriD::verbose;
 	return $g;
 }
 
-sub objplotcommand {
-	my($object) = @_;
-	my $win = PDL::Graphics::TriD::get_current_window();
-	my $world = $win->world();
-}
-
 sub checkargs {
 	if(ref $_[$#_] eq "HASH" and $PDL::Graphics::TriD::verbose) {
 
 	  print "enter checkargs \n";
-		for([KeepTwiddling,\&keeptwiddling3d]) {
+		for(['KeepTwiddling',\&keeptwiddling3d]) {
 		  print "checkargs >$_<\n";
 			if(defined $_[$#_]{$_->[0]}) {
 				&{$_->[1]}(delete $_[$#_]{$_->[0]});
@@ -764,18 +797,22 @@ sub checkargs {
 	}
 }
 
-*keeptwiddling3d = \&PDL::keeptwiddling3d;
+*keeptwiddling3d=*keeptwiddling3d=\&PDL::keeptwiddling3d;
 sub PDL::keeptwiddling3d {
-	$PDL::Graphics::TriD::keeptwiddling = (defined $_[0] ? $_[0] : 1);
+	$PDL::Graphics::TriD::keeptwiddling = $_[0] // 1;
 }
-*nokeeptwiddling3d = \&PDL::nokeeptwiddling3d;
+*nokeeptwiddling3d=*nokeeptwiddling3d=\&PDL::nokeeptwiddling3d;
 sub PDL::nokeeptwiddling3d {
 	$PDL::Graphics::TriD::keeptwiddling = 0 ;
 }
 keeptwiddling3d();
-*twiddle3d = \&PDL::twiddle3d;
-sub PDL::twiddle3d {
-	twiddle_current();
+*twiddle3d = *twiddle3d = *PDL::twiddle3d = \&twiddle_current;
+
+*close3d = *close3d = \&PDL::close3d;
+sub PDL::close3d {
+  return if !ref $PDL::Graphics::TriD::current_window;
+  return if !$PDL::Graphics::TriD::current_window->can('close');
+  $PDL::Graphics::TriD::current_window->close;
 }
 
 sub graph_object {
@@ -783,40 +820,36 @@ sub graph_object {
 	if(!defined $obj or !ref $obj) {
 		barf("Invalid object to TriD::graph_object");
 	}
-	print "graph_object: calling get_new_graph\n" if($PDL::debug_trid);
+	print "graph_object: calling get_new_graph\n" if($PDL::Graphics::TriD::verbose);
 	my $g = get_new_graph();
-	print "graph_object: back from get_new_graph\n" if($PDL::debug_trid);
-
+	print "graph_object: back from get_new_graph\n" if($PDL::Graphics::TriD::verbose);
 	my $name = $g->add_dataseries($obj);
 	$g->bind_default($name);
 	$g->scalethings();
 	print "ADDED TO GRAPH: '$name'\n" if $PDL::Graphics::TriD::verbose;
-
 	twiddle_current();
 	return $obj;
 }
 
 # Plotting routines that use the whole viewport
 
-*describe3d=\&PDL::describe3d;
+*describe3d=*describe3d=\&PDL::describe3d;
 sub PDL::describe3d {
 	require PDL::Graphics::TriD::TextObjects;
 	my ($text) = @_;
 	my $win = PDL::Graphics::TriD::get_current_window();
-	my $imag = new PDL::Graphics::TriD::Description($text);
+	my $imag = PDL::Graphics::TriD::Description->new($text);
 	$win->add_object($imag);
 #	$win->twiddle();
 }
 
-*imagrgb=\&PDL::imagrgb;
+*imagrgb=*imagrgb=\&PDL::imagrgb;
 sub PDL::imagrgb {
 	require PDL::Graphics::TriD::Image;
 	my (@data) = @_; &checkargs;
 	my $win = PDL::Graphics::TriD::get_current_window();
-	my $imag = new PDL::Graphics::TriD::Image(@data);
-
+	my $imag = PDL::Graphics::TriD::Image->new(@data);
 	$win->clear_viewports();
-
 	$win->current_viewport()->add_object($imag);
 	$win->twiddle();
 }
@@ -824,99 +857,90 @@ sub PDL::imagrgb {
 # Plotting routines that use the 3D graph
 
 # Call: line3d([$x,$y,$z],[$color]);
-*line3d=\&PDL::line3d;
-sub PDL::line3d { 
+*line3d=*line3d=\&PDL::line3d;
+sub PDL::line3d {
     &checkargs;
-    my $obj = new PDL::Graphics::TriD::LineStrip(@_);
-    print "line3d: object is $obj\n" if($PDL::debug_trid);
-    &graph_object($obj);
+    my $obj = PDL::Graphics::TriD::LineStrip->new(@_);
+    print "line3d: object is $obj\n" if($PDL::Graphics::TriD::verbose);
+    graph_object($obj);
 }
 
-*contour3d=\&PDL::contour3d;
-sub PDL::contour3d { 
+*contour3d=*contour3d=\&PDL::contour3d;
+sub PDL::contour3d {
 #  &checkargs;
   require PDL::Graphics::TriD::Contours;
-  &graph_object(new PDL::Graphics::TriD::Contours(@_));
+  graph_object(PDL::Graphics::TriD::Contours->new(@_));
 }
 
 # XXX Should enable different positioning...
-*imagrgb3d=\&PDL::imagrgb3d;
+*imagrgb3d=*imagrgb3d=\&PDL::imagrgb3d;
 sub PDL::imagrgb3d { &checkargs;
 	require PDL::Graphics::TriD::Image;
-	&graph_object(new PDL::Graphics::TriD::Image(@_));
+	graph_object(PDL::Graphics::TriD::Image->new(@_));
 }
 
-*imag3d_ns=\&PDL::imag3d_ns;
+*imag3d_ns=*imag3d_ns=\&PDL::imag3d_ns;
 sub PDL::imag3d_ns {  &checkargs;
-	&graph_object(new PDL::Graphics::TriD::SLattice(@_));
+	graph_object(PDL::Graphics::TriD::SLattice->new(@_));
 }
 
-*imag3d=\&PDL::imag3d;
+*imag3d=*imag3d=\&PDL::imag3d;
 sub PDL::imag3d { &checkargs;
-	&graph_object(new PDL::Graphics::TriD::SLattice_S(@_));
+	graph_object(PDL::Graphics::TriD::SLattice_S->new(@_));
 }
 
-####################################################################
-################ JNK 15mar11 added section start ###################
-*STrigrid_S_imag3d=\&PDL::STrigrid_S_imag3d;
-sub PDL::STrigrid_S_imag3d { &checkargs;
-  &graph_object(new PDL::Graphics::TriD::STrigrid_S(@_)); }
-        
-*STrigrid_imag3d=\&PDL::STrigrid_imag3d;
-sub PDL::STrigrid_imag3d { &checkargs;
-  &graph_object(new PDL::Graphics::TriD::STrigrid(@_)); }
-################ JNK 15mar11 added section finis ###################
-####################################################################
+*trigrid3d=*trigrid3d=\&PDL::trigrid3d;
+sub PDL::trigrid3d { &checkargs;
+  graph_object(PDL::Graphics::TriD::STrigrid_S->new(@_)); }
 
-*mesh3d=\&PDL::mesh3d;
-*lattice3d=\&PDL::mesh3d;
-*PDL::lattice3d=\&PDL::mesh3d;
+*trigrid3d_ns=*trigrid3d_ns=\&PDL::trigrid3d_ns;
+sub PDL::trigrid3d_ns { &checkargs;
+  graph_object(PDL::Graphics::TriD::STrigrid->new(@_)); }
+
+*mesh3d=*mesh3d=\&PDL::mesh3d;
+*lattice3d=*lattice3d=\&PDL::mesh3d;
+*PDL::lattice3d=*PDL::lattice3d=\&PDL::mesh3d;
 sub PDL::mesh3d { &checkargs;
-	&graph_object(new PDL::Graphics::TriD::Lattice(@_));
+	graph_object(PDL::Graphics::TriD::Lattice->new(@_));
 }
 
-*points3d=\&PDL::points3d;
+*points3d=*points3d=\&PDL::points3d;
 sub PDL::points3d { &checkargs;
-	&graph_object(new PDL::Graphics::TriD::Points(@_));
+	graph_object(PDL::Graphics::TriD::Points->new(@_));
 }
 
-
-*spheres3d=\&PDL::spheres3d;
+*spheres3d=*spheres3d=\&PDL::spheres3d;
 sub PDL::spheres3d { &checkargs;
-	&graph_object(new PDL::Graphics::TriD::Spheres(@_));
+	graph_object(PDL::Graphics::TriD::Spheres->new(@_));
 }
 
-*grabpic3d=\&PDL::grabpic3d;
+*grabpic3d=*grabpic3d=\&PDL::grabpic3d;
 sub PDL::grabpic3d {
 	my $win = PDL::Graphics::TriD::get_current_window();
-	barf "backend doesn't support grabing the rendered scene"
+	barf "backend doesn't support grabbing the rendered scene"
 	  unless $win->can('read_picture');
 	my $pic = $win->read_picture();
 	return ($pic->float) / 255;
 }
 
-$PDL::Graphics::TriD::hold_on = 0;
+$PDL::Graphics::TriD::only_one = 1;
+sub PDL::hold3d {$PDL::Graphics::TriD::only_one = !($_[0] // 1);}
+sub PDL::release3d {$PDL::Graphics::TriD::only_one = 1;}
 
-sub PDL::hold3d {$PDL::Graphics::TriD::hold_on =(!defined $_[0] ? 1 : $_[0]);}
-sub PDL::release3d {$PDL::Graphics::TriD::hold_on = 0;}
-
-*hold3d=\&PDL::hold3d;
-*release3d=\&PDL::release3d;
+*hold3d=*hold3d=\&PDL::hold3d;
+*release3d=*release3d=\&PDL::release3d;
 
 sub get_new_graph {
-    print "get_new_graph: calling PDL::Graphics::TriD::get_current_window...\n" if($PDL::debug_trid);
+    print "get_new_graph: calling PDL::Graphics::TriD::get_current_window...\n" if($PDL::Graphics::TriD::verbose);
 	my $win = PDL::Graphics::TriD::get_current_window();
-
-    print "get_new_graph: calling get_current_graph...\n" if($PDL::debug_trid);
+    print "get_new_graph: calling get_current_graph...\n" if($PDL::Graphics::TriD::verbose);
 	my $g = get_current_graph($win);
-    print "get_new_graph: back get_current_graph returned $g...\n" if($PDL::debug_trid);
-
-	if(!$PDL::Graphics::TriD::hold_on) {
-		$g->clear_data();
-		$win->clear_viewport();
+    print "get_new_graph: back get_current_graph returned $g...\n" if($PDL::Graphics::TriD::verbose);
+	if ($PDL::Graphics::TriD::only_one) {
+		$g->clear_data;
+		$win->clear_viewport;
 	}
-	$g->default_axes();
-
+	$g->default_axes;
 	$win->add_object($g);
 	return $g;
 }
@@ -924,53 +948,33 @@ sub get_new_graph {
 sub get_current_graph {
    my $win = shift;
 	my $g = $win->current_viewport()->graph();
-
 	if(!defined $g) {
-		$g = new PDL::Graphics::TriD::Graph();
+		$g = PDL::Graphics::TriD::Graph->new;
 		$g->default_axes();
 		$win->current_viewport()->graph($g);
 	}
 	return $g;
 }
 
-
-# $PDL::Graphics::TriD::cur = {};
 # $PDL::Graphics::TriD::create_window_sub = undef;
 sub get_current_window {
   my $opts = shift @_;
-  my $win = $PDL::Graphics::TriD::cur;
+  my $win = $PDL::Graphics::TriD::current_window;
 
   if(!defined $win) {
 	 if(!$PDL::Graphics::TriD::create_window_sub) {
 		barf("PDL::Graphics::TriD must be used with a display mechanism: for example PDL::Graphics::TriD::GL!\n");
 	 }
-	 print "get_current_window - creating window...\n" if($PDL::debug_trid);
-	 $win = new PDL::Graphics::TriD::Window($opts);
+	 print "get_current_window - creating window...\n" if($PDL::Graphics::TriD::verbose);
+	 $PDL::Graphics::TriD::current_window = $win = PDL::Graphics::TriD::Window->new($opts);
 
-	 print "get_current_window - calling set_material...\n" if($PDL::debug_trid);
-	 $win->set_material(new PDL::Graphics::TriD::Material);
-	 $PDL::Graphics::TriD::current_window = $win;
-	 $PDL::Graphics::TriD::cur = $win
+	 print "get_current_window - calling set_material...\n" if($PDL::Graphics::TriD::verbose);
+	 $win->set_material(PDL::Graphics::TriD::Material->new);
   }
   return $PDL::Graphics::TriD::current_window;
 }
 
-# Get the current graphbox
-sub get_current_graphbox {
-        die "get_current_graphbox: ERROR graphbox is not implemented! \n";
-	my $graph = $PDL::Graphics::TriD::curgraph;
-	if(!defined $graph) {
-		$graph = new PDL::Graphics::TriD::Graph();
-		$graph->default_axes();
-		$PDL::Graphics::TriD::curgraph = $graph;
-	}
-	return $graph;
-}
-
-sub twiddle_current {
-	my $win = get_current_window();
-	$win->twiddle();
-}
+sub twiddle_current { get_current_window()->twiddle() }
 
 ###################################
 #
@@ -998,7 +1002,7 @@ package PDL::Graphics::TriD::BoundingBox;
 use base qw/PDL::Graphics::TriD::Object/;
 use fields qw/Box/;
 
-sub new { 
+sub new {
   my($type,$x0,$y0,$z0,$x1,$y1,$z1) = @_;
   my $this = $type->SUPER::new();
   $this->{Box} = [$x0,$y0,$z0,$x1,$y1,$z1];
@@ -1020,19 +1024,13 @@ sub normalize {my($this,$x0,$y0,$z0,$x1,$y1,$z1) = @_;
 	return $trans;
 }
 
-
-###################################
-#
-#
 package PDL::Graphics::TriD::OneTransformation;
 use fields qw/Args/;
 
 sub new {
   my($type,@args) = @_;
   my $this = fields::new($type);
-
   $this->{Args} = [@args];
-
   $this;
 }
 
@@ -1046,22 +1044,11 @@ use base qw/PDL::Graphics::TriD::OneTransformation/;
 package PDL::Graphics::TriD::Transformation;
 use base qw/PDL::Graphics::TriD::Object/;
 
-#sub new {
-#	my($type) = @_;
-#	bless {},$type;
-#}
-
 sub add_transformation {
 	my($this,$trans) = @_;
 	push @{$this->{Transforms}},$trans;
 }
 
-
-
-=head1 BUGS
-
-Not enough is there yet.
-
 =head1 AUTHOR
 
 Copyright (C) 1997 Tuomas J. Lukka (lukka@husc.harvard.edu). Documentation
@@ -1072,5 +1059,6 @@ conditions. For details, see the file COPYING in the PDL
 distribution. If this file is separated from the PDL distribution,
 the copyright notice should be included in the file.
 
-
 =cut
+
+1;
